package com.ehsure.scanner_view;

import android.content.Context;
import android.content.pm.PackageManager;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;
import me.dm7.barcodescanner.zbar.BarcodeFormat;
import me.dm7.barcodescanner.zbar.Result;
import me.dm7.barcodescanner.zbar.ZBarScannerView;

public class ScannerView implements PlatformView, ZBarScannerView.ResultHandler {

    private final ZBarScannerView mZBarScannerView;
    private final View scannerLayout;
    private MethodChannel methodChannel;

    ScannerView(Context context, BinaryMessenger messenger, int id, Map<String, Object> params) {
        methodChannel = new MethodChannel(messenger, "ehsure_scanner_channel");
        boolean hasFlash = context.getPackageManager().hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH);
        methodChannel.setMethodCallHandler(new MethodChannel.MethodCallHandler(){
            @Override
            public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
                if (call.method.equals("scanResume")) {
                    mZBarScannerView.startCamera();
                } else {
                    result.notImplemented();
                }
            }
        });
        scannerLayout = LayoutInflater.from(context).inflate(R.layout.activity_er_scanner, null);
        mZBarScannerView = scannerLayout.findViewById(R.id.view_ZBarScanner);
        ImageView ivFlashlight = scannerLayout.findViewById(R.id.iv_flashlight);
        ivFlashlight.setOnClickListener(v -> {
            if (hasFlash) {
                boolean flash = mZBarScannerView.getFlash();
                if (flash) {
                    ivFlashlight.setImageResource(R.mipmap.ic_flashlight_off);
                } else {
                    ivFlashlight.setImageResource(R.mipmap.ic_flashlight_on);
                }
                mZBarScannerView.setFlash(!flash);
            }
        });

        mZBarScannerView.setSquareViewFinder(true);
        List<BarcodeFormat> formats = new ArrayList<>();
        formats.add(BarcodeFormat.QRCODE);
        mZBarScannerView.setFormats(formats);
        mZBarScannerView.startCamera();
        mZBarScannerView.setResultHandler(this);
    }

    @Override
    public View getView() {
        return scannerLayout;
    }

    @Override
    public void dispose() {
        methodChannel.setMethodCallHandler(null);
        mZBarScannerView.stopCamera();
    }

    @Override
    public void handleResult(Result rawResult) {
        methodChannel.invokeMethod("scanResult", rawResult.getContents(), new MethodChannel.Result(){
            @Override
            public void success(@Nullable Object result) {
                if (result != null) {
                    mZBarScannerView.resumeCameraPreview(ScannerView.this);
                }
            }

            @Override
            public void error(String errorCode, @Nullable String errorMessage, @Nullable Object errorDetails) {
                Log.e(errorCode + errorMessage, errorDetails.toString());
            }

            @Override
            public void notImplemented() {

            }
        });
    }
}