import 'dart:async';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

typedef EhsureScannerCallback = FutureOr<void> Function(String scanResult);
class ScannerView {
  static const String SCANNER_METHOD_CHANNEL = "ehsure_scanner_channel";
  static const String SCANNER_METHOD = "scanResult";
  static const String SCANNER_RESUME_METHOD = "scanResume";
  static const String SCANNER_VIEW_TYPE_ID = "ehsure_scanner_view";
  static const String SCANNER_OPEN_FLASH = "open_flash";
  static const String SCANNER_CLOSE_FLASH = "close_flash";
  // await channel.invokeMethod('open');

  static Future<void> getScanResult (BuildContext context, {@required EhsureScannerCallback callback, bool isSingle = true}) async {
      await Navigator.push(context, MaterialPageRoute(builder: (context) {
        return PlatformScannerView(callback: callback, isSingle: isSingle,);
      }));
  }
}

class PlatformScannerView extends StatefulWidget {
  final EhsureScannerCallback callback;
  final bool isSingle;
  PlatformScannerView({Key key, this.callback, this.isSingle}): super(key: key);
  @override
  _PlatformScannerViewState createState() => _PlatformScannerViewState();
}

class _PlatformScannerViewState extends State<PlatformScannerView> {

  final MethodChannel channel = MethodChannel(ScannerView.SCANNER_METHOD_CHANNEL);

  @override
  void initState() {
    super.initState();
    handleChannel();
  }

  handleChannel() {
    channel.setMethodCallHandler((MethodCall call) async {
      if (call?.method == ScannerView.SCANNER_METHOD) {
        await widget.callback(call.arguments.toString());
        if (widget.isSingle) {
          Navigator.of(context).pop();
          return null;
        }
        if (Platform.isIOS) {
          await setScanResume();
        } else {
          return 'success';
        }
      }
      return null;
    });
  }

  Future<void> setScanResume() async {
    return channel.invokeMethod(ScannerView.SCANNER_RESUME_METHOD);
  }

  //弹出服务器地址框
  Future<String> showInputDialog() {
    String inputString; //服务器地址
    return showDialog<String>(
        context: context,
        builder: (context) {
          return AlertDialog(
            title: Text("手动输入"),
            content: Column(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                Row(
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    Expanded(
                      child: TextField(
                        decoration: InputDecoration(hintText: ""),
                        onChanged: (value) {
                          inputString = value;
                        },
                        keyboardType: TextInputType.url,
                      ),
                    )
                  ],
                ),
              ],
            ),
            actions: <Widget>[
              FlatButton(
                child: Text('取消'),
                onPressed: () {
                  Navigator.of(context).pop();
                },
              ),
              FlatButton(
                child: Text('确定'),
                onPressed: () {
                  Navigator.of(context).pop(inputString);
                },
              )
            ],
          );
        });
  }
  @override
  Widget build(BuildContext context) {
    Widget pView;
    if (Platform.isIOS) {
      pView = UiKitView(
        viewType: ScannerView.SCANNER_VIEW_TYPE_ID,
        gestureRecognizers: <Factory<OneSequenceGestureRecognizer>>[
          new Factory<OneSequenceGestureRecognizer>(
                () => new EagerGestureRecognizer(),
          ),
        ].toSet(),
      );
    } else {
      pView = AndroidView(
        viewType: ScannerView.SCANNER_VIEW_TYPE_ID,
        gestureRecognizers: <Factory<OneSequenceGestureRecognizer>>[
          new Factory<OneSequenceGestureRecognizer>(
                () => new EagerGestureRecognizer(),
          ),
        ].toSet(),
      );
    }
    return Scaffold(
      body: Container(
        child: Stack(
          fit: StackFit.expand,
          children: <Widget>[
            pView,
            Positioned(
              left: 0,
              right: 0,
              top: 100,
              height: 50,
              child: Center(child: FlatButton(
                onPressed: () async {
                  String result = await showInputDialog();
                  if (result != null) {
                    widget.callback(result);
                    if (widget.isSingle) {
                      Navigator.of(context).pop();
                    } else {
                      await setScanResume();
                    }
                  }
                },
                child: Text("手动输入", style: TextStyle(color: Colors.white),),)
              ),
            )
          ],
        ),
      ),
    );
  }
}

